package javax.swing;

import sun.security.action.GetBooleanAction;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import sun.awt.AppContext;
import sun.awt.event.IgnorePaintEvent;
import java.awt.Rectangle;
import java.awt.Container;
import sun.awt.SunToolkit;
import java.awt.event.PaintEvent;
import java.awt.Component;
import sun.awt.PaintEventDispatcher;

class SwingPaintEventDispatcher extends PaintEventDispatcher
{
    private static final boolean SHOW_FROM_DOUBLE_BUFFER;
    private static final boolean ERASE_BACKGROUND;
    
    SwingPaintEventDispatcher() {
    }
    
    @Override
    public PaintEvent createPaintEvent(Component source, int x, int y, int width, int height) {
        if ((source instanceof RootPaneContainer)) {
            AppContext targetToAppContext2 = SunToolkit.targetToAppContext(source);
            RepaintManager currentManager = RepaintManager.currentManager(targetToAppContext2);
            if (!SwingPaintEventDispatcher.SHOW_FROM_DOUBLE_BUFFER || !currentManager.show((Container)source, x, y, width, height)) {
                currentManager.nativeAddDirtyRegion(targetToAppContext2, (Container)source, x, y, width, height);
            }
            return new IgnorePaintEvent(source, 800, new Rectangle(x, y, width, height));
        }
        else {
            if ((source instanceof SwingHeavyWeight)) {
                AppContext targetToAppContext = SunToolkit.targetToAppContext(source);
                RepaintManager.currentManager(targetToAppContext).nativeAddDirtyRegion(targetToAppContext, (Container)source, x, y, width, height);
                return new IgnorePaintEvent(source, 800, new Rectangle(x, y, width, height));
            }
            return super.createPaintEvent(source, x, y, width, height);
        }
    }
    
    @Override
    public boolean shouldDoNativeBackgroundErase(Component component) {
        return SwingPaintEventDispatcher.ERASE_BACKGROUND || !(component instanceof RootPaneContainer);
    }
    
    @Override
    public boolean queueSurfaceDataReplacing(Component component, Runnable r) {
        if ((component instanceof RootPaneContainer)) {
            AppContext targetToAppContext = SunToolkit.targetToAppContext(component);
            RepaintManager.currentManager(targetToAppContext).nativeQueueSurfaceDataRunnable(targetToAppContext, component, r);
            return true;
        }
        return super.queueSurfaceDataReplacing(component, r);
    }
    
    static {
        SHOW_FROM_DOUBLE_BUFFER = "true".equals(AccessController.doPrivileged(new GetPropertyAction("swing.showFromDoubleBuffer", "true")));
        ERASE_BACKGROUND = (boolean)AccessController.doPrivileged(new GetBooleanAction("swing.nativeErase"));
    }
}
